﻿using AutoMapper;
using BizService;
using Model.Dto;
using Model.Entities;
using PLCTool.Common;
using Prism.Mvvm;
using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media.Animation;

namespace PLCTool.ViewModels.MonitorManageViewModel
{
    public  class PLCReadViewModel:BindableBase
    {
        private readonly IBasicPlcItemConfigService _iBasPlcItemConfigService;

        private readonly IMapper _mapper;
        private List<BasPlcItemConfigDto> plcConfigs = new List<BasPlcItemConfigDto>();
        public PLCReadViewModel(IBasicPlcItemConfigService iBasPlcItemConfigService, IMapper mapper)
        {
          
            _iBasPlcItemConfigService = iBasPlcItemConfigService;
            _mapper = mapper;
            GetPLCConfig();
            ReadPLCRealValue();
        }
        #region 私有方法
        /// <summary>
        /// 获取PLC配置
        /// </summary>
        private void GetPLCConfig()
        {
            var plclist = _iBasPlcItemConfigService.QueryList();
            plcConfigs = _mapper.Map<List<bas_plc_item_config>, List<BasPlcItemConfigDto>>(plclist);
            foreach (var plc in plcConfigs)
            {
                PLCItemList.Add(plc);
            }
        }

        #endregion
        #region 命令绑定
        private ObservableCollection<BasPlcItemConfigDto> plcItemList=new ObservableCollection<BasPlcItemConfigDto>();
        public ObservableCollection<BasPlcItemConfigDto> PLCItemList
        {
            get { return plcItemList; }
            set { plcItemList = value; RaisePropertyChanged(); }
        }
        private void ReadPLCRealValue()
        {
                Task.Run(async () =>
                {
                    while(true)
                    {
                        //更新界面上的值
                        Application.Current.Dispatcher.Invoke(() =>
                        {
                            foreach (var item in PLCItemList)
                            {
                                //没有连接 plc，直接退出
                                if(!PLCCom.GetInstance().IsConnectPLC())
                                {
                                    break;
                                }
                                string plcAddr = item.PlcAddress;
                                string plcAddType = item.PlcAddType;
                                switch (plcAddType)
                                {
                                    case "bool":
                                        item.RealValue = PLCCom.GetInstance().ReadPlcObject(plcAddr, VarType.Bit);
                                        break;
                                    case "word":
                                        item.RealValue = PLCCom.GetInstance().ReadPlcObject(plcAddr, VarType.Word);
                                        break;
                                    case "dword":
                                        item.RealValue = PLCCom.GetInstance().ReadPlcObject(plcAddr, VarType.DWord);
                                        break;
                                    case "real":
                                        item.RealValue = PLCCom.GetInstance().ReadPlcObject(plcAddr, VarType.Real);
                                        break;
                                }

                            }
                        });

                        await Task.Delay(5000);
                    }

                });
        }
        #endregion
  
    }
}
